"""Filtering logic for Tunnel instances.

(c) 2020 Justin Drew
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at
  http://www.apache.org/licenses/LICENSE-2.0
Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

import django_filters
from django.db.models import Q

from utilities.filters import NameSlugSearchFilterSet

from .models import Tunnel


class TunnelFilter(NameSlugSearchFilterSet):
    """Filter capabilities for Tunnel instances."""

    q = django_filters.CharFilter(method="search", label="Search",)

    name = django_filters.ModelMultipleChoiceFilter(
        field_name="name__slug", queryset=Tunnel.objects.all(), to_field_name="slug", label="Tunnel Name (slug)",
    )

    status = django_filters.ModelMultipleChoiceFilter(
        field_name="status__slug", queryset=Tunnel.objects.all(), to_field_name="slug", label="Tunnel Status (slug)",
    )

    tunnel_type = django_filters.ModelMultipleChoiceFilter(
        field_name="tunnel_type__slug", queryset=Tunnel.objects.all(), to_field_name="slug", label="Tunnel Type (slug)",
    )

    class Meta:
        """Class to define what is used for filtering tunnels with the search box."""

        model = Tunnel
        fields = ["name", "status", "tunnel_type"]

    def search(self, queryset, name, value):
        """Perform the filtered search."""
        if not value.strip():
            return queryset
        qs_filter = Q(name__icontains=value) | Q(status__icontains=value) | Q(tunnel_type__icontains=value)
        return queryset.filter(qs_filter)



"""Plugin declaration for netbox_tunnels.

(c) 2020 Justin Drew
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at
  http://www.apache.org/licenses/LICENSE-2.0
Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

__version__ = "0.3.3"

from extras.plugins import PluginConfig


class TunnelsConfig(PluginConfig):
    """This class defines attributes for the NetBox Tunnels Plugin."""

    name = "netbox_tunnels"
    verbose_name = "Network Tunnels"
    version = __version__
    description = "Network Tunnels"
    base_url = "netbox_tunnels"
    author = "Justin Drew"
    author_email = "justin.drew@networktocode.com"
    min_version = "2.8.3"
    required_settings = []
    # default_settings = {}
    # caching_config = {}


config = TunnelsConfig  # pylint:disable=invalid-name



# Generated by Django 3.0.6 on 2020-05-27 01:49

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ("dcim", "0105_interface_name_collation"),
    ]

    operations = [
        migrations.CreateModel(
            name="Tunnel",
            fields=[
                ("tunnel_id", models.AutoField(primary_key=True, serialize=False)),
                ("name", models.CharField(max_length=64)),
                ("status", models.CharField(default="pending-configuration", max_length=30)),
                ("tunnel_type", models.CharField(default="ipsec-tunnel", max_length=30)),
                ("src_address", models.CharField(blank=True, max_length=28)),
                ("dst_address", models.CharField(blank=True, max_length=28)),
                ("psk", models.CharField(blank=True, max_length=100)),
            ],
            options={"ordering": ["tunnel_id"],},
        ),
        migrations.CreateModel(
            name="TunnelDevice",
            fields=[
                ("id", models.AutoField(auto_created=True, primary_key=True, serialize=False)),
                (
                    "device",
                    models.OneToOneField(
                        on_delete=django.db.models.deletion.CASCADE, related_name="device_of", to="dcim.Device"
                    ),
                ),
                (
                    "tunnel",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE, related_name="device", to="netbox_tunnels.Tunnel"
                    ),
                ),
            ],
            options={"ordering": ["tunnel"],},
        ),
    ]



"""Forms for tunnel creation.

(c) 2020 Justin Drew
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at
  http://www.apache.org/licenses/LICENSE-2.0
Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

from django import forms

from utilities.forms import BootstrapMixin
from dcim.models import Device
from extras.forms import CustomFieldModelCSVForm

from .models import Tunnel
from .choices import TunnelStatusChoices, TunnelTypeChoices

BLANK_CHOICE = (("", "---------"),)


class TunnelCreationForm(BootstrapMixin, forms.ModelForm):
    """Form for creating a new tunnel."""

    name = forms.CharField(required=True, label="Name", help_text="Name of tunnel")

    status = forms.ChoiceField(choices=BLANK_CHOICE + TunnelStatusChoices.CHOICES, required=False)

    tunnel_type = forms.ChoiceField(
        choices=TunnelTypeChoices.CHOICES,
        required=True,
        label="Tunnel type",
        help_text="Tunnel type. This must be specified.",
    )

    src_address = forms.CharField(required=True, label="Source IP address", help_text="IP address of the source device")

    dst_address = forms.CharField(required=True, label="Peer IP address", help_text="IP address of the peer device")

    psk = forms.CharField(
        required=False, label="Pre-shared Key", widget=forms.PasswordInput, help_text="Pre-shared key"
    )

    class Meta:
        """Class to define what is used to create a new network tunnel."""

        model = Tunnel
        fields = [
            "name",
            "status",
            "tunnel_type",
            "src_address",
            "dst_address",
            "psk",
        ]


class TunnelFilterForm(BootstrapMixin, forms.ModelForm):
    """Form for filtering Tunnel instances."""

    device = forms.ModelChoiceField(queryset=Device.objects.all(), required=False)

    status = forms.ChoiceField(choices=BLANK_CHOICE + TunnelStatusChoices.CHOICES, required=False)

    q = forms.CharField(required=False, label="Search")

    class Meta:
        """Class to define what is used for filtering tunnels with the search box."""

        model = Tunnel
        fields = [
            "src_address",
            "dst_address",
            "psk",
            "tunnel_type",
        ]


class TunnelCreationCSVForm(CustomFieldModelCSVForm):
    """Form for entering CSV to bulk-import Tunnel entries."""

    src_address = forms.CharField(required=True, help_text="IP Address of the source device")
    dst_address = forms.CharField(required=True, help_text="IP Address of the peer device")
    tunnel_type = forms.CharField(required=True, help_text="Specified tunnel type.")
    psk = forms.CharField(required=True, help_text="Pre-shared key")

    class Meta:
        """Class to define what is used for bulk import of tunnels form using CSV."""

        model = Tunnel
        fields = Tunnel.csv_headers

    def save(self, commit=True, **kwargs):
        """Save the model, and add it and the associated PSK."""
        model = super().save(commit=commit, **kwargs)
        # if commit:
        #     credentials = Credentials(self.data.get("psk"))
        return model



"""Plugin additions to the NetBox navigation menu.

(c) 2020 Justin Drew
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at
  http://www.apache.org/licenses/LICENSE-2.0
Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

from extras.plugins import PluginMenuButton, PluginMenuItem
from utilities.choices import ButtonColorChoices


menu_items = (
    PluginMenuItem(
        link="plugins:netbox_tunnels:tunnels_list",
        link_text="Tunnels",
        permissions=["netbox_tunnels.view_tunnels"],
        buttons=(
            # Link to the plugins view to add a tunnel if user has "add_tunnels" permission.
            PluginMenuButton(
                link="plugins:netbox_tunnels:tunnel_creation",
                title="Add a new tunnel",
                icon_class="fa fa-plus",
                color=ButtonColorChoices.GREEN,
                permissions=["netbox_tunnels.add_tunnels"],
            ),
            # Links to the plugins view to bulk import tunnels if user has the "add_tunnels" permission.
            PluginMenuButton(
                link="plugins:netbox_tunnels:tunnels_import",
                title="Bulk import tunnels",
                icon_class="fa fa-download",
                color=ButtonColorChoices.BLUE,
                permissions=["netbox_tunnels.add_tunnels"],
            ),
        ),
    ),
)



"""Django views for network tunnels.

(c) 2020 Justin Drew
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at
  http://www.apache.org/licenses/LICENSE-2.0
Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""
from django.contrib.auth.mixins import PermissionRequiredMixin
from utilities.views import BulkDeleteView, BulkImportView, ObjectEditView, ObjectListView

from .filters import TunnelFilter
from .forms import TunnelCreationForm, TunnelFilterForm, TunnelCreationCSVForm
from .models import Tunnel
from .tables import TunnelTable, TunnelBulkTable


class ListTunnelView(PermissionRequiredMixin, ObjectListView):
    """View for listing all Tunnels."""

    permission_required = "netbox_tunnels.view_tunnels"
    model = Tunnel
    queryset = Tunnel.objects.all().order_by("tunnel_id")
    filterset = TunnelFilter
    filterset_form = TunnelFilterForm
    table = TunnelTable
    template_name = "netbox_tunnels/tunnels_list.html"


class CreateTunnelView(PermissionRequiredMixin, ObjectEditView):
    """View for creating a new Tunnels."""

    permission_required = "netbox_tunnels.add_tunnels"
    model = Tunnel
    queryset = Tunnel.objects.all()
    model_form = TunnelCreationForm
    template_name = "netbox_tunnels/tunnel_edit.html"
    default_return_url = "plugins:netbox_tunnels:tunnels_list"


class BulkDeleteTunnelView(PermissionRequiredMixin, BulkDeleteView):
    """View for deleting one or more Tunnels."""

    permission_required = "netbox_tunnels.delete_tunnels"
    queryset = Tunnel.objects.filter()
    table = TunnelTable
    default_return_url = "plugins:netbox_tunnels:tunnels_list"


class BulkImportTunnelView(PermissionRequiredMixin, BulkImportView):
    """View for bulk-importing a CSV file to create Tunnels."""

    permission_required = "netbox_tunnels.add_tunnels"
    model_form = TunnelCreationCSVForm
    tunnel = TunnelBulkTable
    default_return_url = "plugins:netbox_tunnels:tunnels_list"



"""Tunnel Django model.

(c) 2020 Justin Drew
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at
  http://www.apache.org/licenses/LICENSE-2.0
Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

from django.db import models
from ipam.models import Device
from .choices import TunnelStatusChoices, TunnelTypeChoices


class Tunnel(models.Model):
    """Tunnel model."""

    tunnel_id = models.AutoField(primary_key=True)
    name = models.CharField(max_length=64)
    status = models.CharField(
        max_length=30, choices=TunnelStatusChoices, default=TunnelStatusChoices.STATUS_PENDING_CONFIGURATION
    )
    tunnel_type = models.CharField(max_length=30, choices=TunnelTypeChoices, default=TunnelTypeChoices.IPSEC_TUNNEL)
    src_address = models.CharField(verbose_name="Source Address", max_length=28, blank=True)
    dst_address = models.CharField(verbose_name="Dest Address", max_length=28, blank=True)
    psk = models.CharField(verbose_name="Pre-shared Key", max_length=100, blank=True)

    csv_headers = [
        "name",
        "status",
        "tunnel_type",
        "src_address",
        "dst_address",
        "psk",
    ]

    class Meta:
        """Class to define what will be used to set order based on. Will be using the unique tunnel ID for this purpose."""

        ordering = ["tunnel_id"]

    def __str__(self):
        """Class to define what identifies the Tunnel object. Will be using name for this."""
        return self.name


class TunnelDevice(models.Model):
    """Tunnel to Device relationship."""

    tunnel = models.ForeignKey(to=Tunnel, on_delete=models.CASCADE, related_name="device")
    device = models.OneToOneField(to=Device, on_delete=models.CASCADE, related_name="device_of")

    class Meta:
        """Class to define what will be used to set order based on. Will be using the unique tunnel for this purpose."""

        ordering = ["tunnel"]



"""Template view for devices.

(c) 2020 Justin Drew
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at
  http://www.apache.org/licenses/LICENSE-2.0
Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

# from extras.plugins import PluginTemplateExtension
# from .models import Tunnel
#
#
# class DeviceTunnels(PluginTemplateExtension):
#     """Class to display tunnels for a device."""
#
#     model = "dcim.devices"
#
#     def full_width_page(self):
#         """Render content of device_tunnel_list on device page."""
#         return self.render("netbox_tunnels/device_tunnel_list.html", extra_context={"tunnels": Tunnel.objects.all(),})
#
#
# template_extensions = [DeviceTunnels]



"""Tables for displaying list of configured Tunnels.

(c) 2020 Justin Drew
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at
  http://www.apache.org/licenses/LICENSE-2.0
Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""
import django_tables2 as tables
from utilities.tables import BaseTable, ToggleColumn
from .models import Tunnel


class TunnelTable(BaseTable):
    """Table for displaying configured Tunnel instances."""

    pk = ToggleColumn()

    class Meta(BaseTable.Meta):
        """Class to define what is used for tunnl_lists.html template to show configured tunnels."""

        model = Tunnel
        fields = [
            "pk",
            "name",
            "status",
            "tunnel_type",
            "src_address",
            "dst_address",
        ]


class TunnelBulkTable(BaseTable):
    """Table for displaying Tunnel imports."""

    pk = tables.LinkColumn()

    class Meta(BaseTable.Meta):
        """Class to define what is used for bulk import of tunnels."""

        model = Tunnel
        fields = (
            "pk",
            "name",
            "status",
            "tunnel_type",
            "src_address",
            "dst_address",
        )



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['netbox_tunnels', 'netbox_tunnels.migrations']

package_data = \
{'': ['*'], 'netbox_tunnels': ['templates/netbox_tunnels/*']}

install_requires = \
['invoke>=1.4.1,<2.0.0']

setup_kwargs = {
    'name': 'netbox-tunnels-plugin',
    'version': '0.3.4',
    'description': 'A plugin for Netbox to document network tunnels',
    'long_description': '# Netbox Tunnels Plugin\n\n<!-- Build status with linky to the builds for ease of access. -->\n[![Build Status](https://travis-ci.com/jdrew82/netbox-tunnels-plugin.svg?token=XHesDxGFcPtaq1Q3URi5&branch=master)](https://travis-ci.com/jdrew82/netbox-tunnels-plugin)\n\n<!-- PyPI version badge. -->\n[![PyPI version](https://badge.fury.io/py/netbox-tunnels-plugin.svg)](https://badge.fury.io/py/netbox-tunnels-plugin)\n\n+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++\n+NOTE: Please be aware that this plugin is still a work in progress and should not be used for production work at this time!+\n+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++\n\nA plugin for [NetBox](https://github.com/netbox-community/netbox) to support documentation of network tunneling\n protocols, ie IPsec, GRE, L2TP, etc.\n\n## Installation\n\nThe plugin is available as a Python package in pypi and can be installed with pip\n```shell\npip install netbox-tunnels-plugin\n```\n\nOnce installed, the plugin needs to be enabled in your `configuration.py`\n```python\n# In your configuration.py\nPLUGINS = ["netbox-tunnels-plugin"]\n\n# PLUGINS_CONFIG = {\n#   "netbox-tunnels-plugin": {\n#     ADD YOUR SETTINGS HERE\n#   }\n# }\n```\n\n## Contributing\n\nPull requests are welcomed and automatically built and tested against multiple version of Python and multiple version of NetBox through TravisCI.\n\nThe project is packaged with a light development environment based on `docker-compose` to help with the local development of the project and to run the tests within TravisCI.\n\nThe project is following Network to Code software development guideline and is leveraging:\n- Black, Pylint, Bandit and pydocstyle for Python linting and formatting.\n- Django unit test to ensure the plugin is working properly.\n\n### CLI Helper Commands\n\nThe project is coming with a CLI helper based on [invoke](http://www.pyinvoke.org/) to help setup the development environment. The commands are listed below in 3 categories `dev environment`, `utility` and `testing`. \n\nEach command can be executed with `invoke <command>`. All commands support the arguments `--netbox-ver` and `--python-ver` if you want to manually define the version of Python and NetBox to use. Each command also has its own help `invoke <command> --help`\n\n#### Local dev environment\n```\n  build            Build all docker images.\n  debug            Start NetBox and its dependencies in debug mode.\n  destroy          Destroy all containers and volumes.\n  start            Start NetBox and its dependencies in detached mode.\n  stop             Stop NetBox and its dependencies.\n```\n\n#### Utility \n```\n  cli              Launch a bash shell inside the running NetBox container.\n  create-user      Create a new user in django (default: admin), will prompt for password.\n  makemigrations   Run Make Migration in Django.\n  nbshell          Launch a nbshell session.\n```\n#### Testing \n\n```\n  tests            Run all tests for this plugin.\n  pylint           Run pylint code analysis.\n  pydocstyle       Run pydocstyle to validate docstring formatting adheres to NTC defined standards.\n  bandit           Run bandit to validate basic static code security analysis.\n  black            Run black to check that Python files adhere to its style standards.\n  unittest         Run Django unit tests for the plugin.\n```\n',
    'author': 'Justin Drew',
    'author_email': 'justin.drew@networktocode.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/jdrew82/netbox-tunnels-plugin',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)
